require_gem 'fxruby'
require 'time'
require 'TimeClock'
require 'CsvExportWriter'
require 'XmlExportWriter'
require 'HtmlExportWriter'
require 'SimpleReportFormatter'
require 'ApplicationProperties'

include Fox

module WorkTimer

class ReportDialog < FXDialogBox
  
   attr_reader :startTimeTextField
   attr_reader :endTimeTextField
   attr_reader :theTimeClock
   attr_reader :reportArea
   attr_reader :exportPatternArray
   attr_reader :exportWriterList
   attr_reader :exportFileExtensionList
   attr_reader :reportFormatter

   def doWindowPropsChanged(source, selector, event)
      reportDialogProps = ApplicationProperties.instance.propertyHash["ReportDialogInfo"]
      if self.width != reportDialogProps["width"].to_i or
         self.height != reportDialogProps["height"].to_i then
         reportDialogProps["width"] = self.width.to_s
         reportDialogProps["height"] = self.height.to_s
      end
   end
   
   def initialize(owner, timeClock, startTime, endTime)
      # Invoke base class initialize function first
      super(owner, "Daily Summary Report", DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE, 0, 0, 290, 400)

      # handle window resizes and moves
      connect(SEL_CONFIGURE, method(:doWindowPropsChanged))
      
      reportDialogProps = ApplicationProperties.instance.propertyHash["ReportDialogInfo"]
      
      self.resize(
         reportDialogProps["width"].to_i,
         reportDialogProps["height"].to_i)
      
      @exportPatternArray = 
         [ "Comma Separated Value (*.csv)",
           "XML (*.xml)",
           "HTML (*.html)" ]
      
      @exportFileExtensionList = ["csv", "xml", "html"]
      
      @exportWriterList = 
         [CsvExportWriter.new, XmlExportWriter.new, HtmlExportWriter.new]
         
      @reportFormatter = SimpleReportFormatter.new
      
      @theTimeClock = timeClock
      
      windowContent = 
      FXVerticalFrame.new(
         self, 
         LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,
         0, 0, 0, 0, 0, 0, 0, 0)
      
      #initialize the current activity fields
      reportDatesGroup = 
        FXGroupBox.new(
           windowContent, 
           "Report Dates",
           GROUPBOX_TITLE_LEFT|FRAME_RIDGE|LAYOUT_FILL_X)
           
      reportDatesData = FXMatrix.new(reportDatesGroup, 4, MATRIX_BY_COLUMNS)
      
      FXLabel.new(reportDatesData, "Start Date:", nil, JUSTIFY_RIGHT)
      
      @startTimeTextField = FXTextField.new(reportDatesData, 10, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y)
      
      FXLabel.new(reportDatesData, "End Date:", nil, JUSTIFY_RIGHT)
        
      @endTimeTextField = FXTextField.new(reportDatesData, 10, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y)
      
      startDate = Time.at(startTime)
      endDate   = Time.at(endTime)
      
      @startTimeTextField.text = formatDate(startDate)
      @endTimeTextField.text = formatDate(endDate)
      
      FXButton.new(
        reportDatesGroup, 
        "&Generate Report", 
        nil, 
        self, 
        0,
        FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y
        ).connect(SEL_COMMAND, method(:generateReport))
                  
      @reportArea = FXText.new(windowContent, nil, 0, TEXT_READONLY|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        
      buttons = FXHorizontalFrame.new(
                  windowContent,
                  LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH)
      
      # Close
      closeButton = FXButton.new(
         buttons, 
         "&Close", 
         nil, 
         self, 
         ID_ACCEPT,
         FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)

      # Export 
      FXButton.new(
         buttons, 
         "&Export...", 
         nil, 
         self, 
         0,
         FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y
         ).connect(SEL_COMMAND, method(:exportReport))
            
      closeButton.setDefault
      closeButton.setFocus
   end

   def exportReport(source, selector, event)
      # open up the file selection dialog to allow
      # the user to specify the export file
      begin
            exportDialog = FXFileDialog.new(self, "Select Report Data Export File")
            exportDialog.patternList = @exportPatternArray
            
            exportDialog.directory = ApplicationProperties.instance.propertyHash["DefaultExportDirectory"]["value"]
            exportDialog.currentPattern = ApplicationProperties.instance.propertyHash["DefaultExportType"]["value"].to_i
            
            return false if (exportDialog.execute == 0)
            
            exportFile = exportDialog.filename
            
            ApplicationProperties.instance.propertyHash["DefaultExportDirectory"]["value"] = exportDialog.directory
            ApplicationProperties.instance.propertyHash["DefaultExportType"]["value"] = exportDialog.currentPattern.to_s
                        
            if exportFile.downcase !~ /\.#{@exportFileExtensionList[exportDialog.currentPattern]}$/ then
               exportFile += ".#{@exportFileExtensionList[exportDialog.currentPattern]}"
            end
            
            if File.exists?(exportFile)
               retVal = FXMessageBox::question(
                  self,
                  MBOX_YES_NO,
                  "Overwrite Document",
                  "Overwrite existing document: \n#{exportFile}?")
                  
               if retVal == MBOX_CLICKED_NO then
                  return
               end
            end
            
            doExportReportData(
               exportFile, 
               @exportWriterList[exportDialog.currentPattern])
      rescue
         FXMessageBox::error(
               self,
               MBOX_OK,
               "Export Error",
               "Error occurred while exporting the report data")       
      end
   end
   
   def doExportReportData(filename, exporter)
      # based on the pattern type, export the data in the appropriate
      # format
      begin
         reportDates = getReportRange
      rescue
         FXMessageBox::error(
               self,
               MBOX_OK,
               "Can't Parse Start/End Date",
               "Dates must be in mm/dd/yyyy format")
         return
      end      
      
      exporter.export(filename, @theTimeClock, reportDates[0], reportDates[1])
   end
      
   def getReportRange
         retList = Array.new
         
         startDate = Time.parse(@startTimeTextField.text)
         
         endDate   = Time.parse(@endTimeTextField.text)
         
         retList.push(Time.local(startDate.year, startDate.month, startDate.day).to_i)
         
         retList.push(Time.local(endDate.year, endDate.month, endDate.day).to_i + 3600 * 24)
         
         return retList  
   end
   
   def generateReport(source, selector, event)
     
      begin
         reportRange = getReportRange
         
         @reportFormatter.format(@theTimeClock, reportRange[0],reportRange[1])
         @reportArea.text = 
            @reportFormatter.format(@theTimeClock, reportRange[0],reportRange[1])

      rescue
         FXMessageBox::error(
               self,
               MBOX_OK,
               "Can't Parse Start/End Date",
               "Dates must be in mm/dd/yyyy format")
      end      
   end

   def formatDate(currTime)
      retString = 
         sprintf(
            "%0.2d/%0.2d/#{currTime.year}",
            currTime.month,
            currTime.day)
            
      return retString
   end
end

end