require_gem 'fxruby'
require 'TimeClock'
require 'ActivityDialog'
require 'ReportDialog'
require 'TimeEditorDialog'
require 'ApplicationProperties'
require 'AboutDialog'

include Fox

module WorkTimer

class TimerGui < FXMainWindow
   attr_reader :currentActivity,
               :currentActivityStartTime,
               :hoursWorkedOnCurrentActivity,
               :activityList,
               :currentActivityStartTimeField,
               :defaultActivityIcon,
               :activeTaskIcon,
               :timeClock,
               :updateTimer,
               :timerInterval,
               :currentActivityIndex,
               :appTitle

   def doWindowPropsChanged(source, selector, event)
      workTimeGuiProps = ApplicationProperties.instance.propertyHash["WorkTimeRecorderInfo"]
      if (self.width > 0 and self.height > 0) and
         (self.width != workTimeGuiProps["width"].to_i or
          self.height != workTimeGuiProps["height"].to_i) then
         workTimeGuiProps["width"] = self.width.to_s
         workTimeGuiProps["height"] = self.height.to_s
      elsif self.x != workTimeGuiProps["x"].to_i or
            self.y != workTimeGuiProps["y"].to_i 
         workTimeGuiProps["x"] = self.x.to_s
         workTimeGuiProps["y"] = self.y.to_s
      end
   end
               
   def initialize(theApp)
      @appTitle = "Work Time Recorder"
      
      super(theApp, @appTitle, nil, nil, DECOR_ALL, 0, 0, 250, 350)
      
      begin
         iconFileName = "WorkTimeRecorder.ico"
         iconFile = File.open(iconFileName, "rb")
         self.icon = FXICOIcon.new(theApp, iconFile.read)
      rescue
         puts "Error reading icon file #{iconFileName}"
      end
      
      initializeMenu
      
      initializeWindow
            
      @timeClock = TimeClock.new("ActivityFile.xml", "EventsFile.xml")
      
      populateActivityList
      
      @timerInterval = 60000 # update timer every minute
      
      @updateTimer = self.getApp().addTimeout(@timerInterval,method(:workTimerCallback))

      @defaultActivityIcon = nil
      
      begin
         File.open("DefaultActivityIcon.ico", "rb") {
            |f|
            @defaultActivityIcon = 
               FXICOIcon.new(theApp, f.read)
               
            @defaultActivityIcon.create
         }
      rescue
         puts "Can't open the default activity icon"
      end

      begin
         File.open("ActiveTaskIcon.ico", "rb") {
            |f|
            @activeTaskIcon = 
               FXICOIcon.new(theApp, f.read)
               
            @activeTaskIcon.create
         }
      rescue
         puts "Can't open the default activity icon"
      end

      # make sure any outstanding activities are closed
      if @timeClock.eventList.size > 0 and
         @timeClock.eventList.last.eventType == "Start"
         activityName = @timeClock.eventList.last.eventDescription
         @timeClock.stopActivity(activityName)
      end
      
      if @timeClock.defaultActivity != nil then
         index = @activityList.findItem(@timeClock.defaultActivity)
         if(index != -1)
            @activityList.selectItem(index)
            @activityList.getItem(index).setIcon(@defaultActivityIcon)
            doStartActivity(@timeClock.defaultActivity, index)
         end
      end
   end
         
   def create
      super
      show
   end
   
   def initializeMenu
      menuBar = FXMenuBar.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
      
      #File Menu
      fileMenu = FXMenuPane.new(self)
      FXMenuCommand.new(fileMenu, "New").connect(SEL_COMMAND, method(:doNewEventFile))
      
      FXMenuCommand.new(fileMenu, "Exit").connect(
            SEL_COMMAND,
            method(:exitApplication))
      
      FXMenuTitle.new(menuBar, "&File", nil, fileMenu)
      
      #Edit Menu
      editMenu = FXMenuPane.new(self)
      #Activity ->Add/Edit/Delete
      activitySubMenu = FXMenuPane.new(self)
      FXMenuCommand.new(activitySubMenu, "Add").connect(SEL_COMMAND, method(:addActivity))
      FXMenuCommand.new(activitySubMenu, "Edit").connect(SEL_COMMAND, method(:editActivity))
      FXMenuCommand.new(activitySubMenu, "Delete").connect(SEL_COMMAND, method(:deleteActivity))
      FXMenuCascade.new(editMenu, "Activity", nil, activitySubMenu)
      
      #Default Activity...
      FXMenuCommand.new(editMenu, "Set Default Activity").connect(SEL_COMMAND, method(:setDefaultActivity))
      
      FXMenuTitle.new(menuBar, "&Edit", nil, editMenu)
      
      #Actions Menu
      actionsMenu = FXMenuPane.new(self)
      
      #Current Activity -> Start, Stop
      currentActivitySubMenu = FXMenuPane.new(self)
      FXMenuCommand.new(currentActivitySubMenu, "Start").connect(
            SEL_COMMAND,
            method(:startActivity))
      FXMenuCommand.new(currentActivitySubMenu, "Stop").connect(
            SEL_COMMAND,
            method(:stopActivity))

      FXMenuCascade.new(actionsMenu, "Current Activity", nil, currentActivitySubMenu)
 
      FXMenuCommand.new(actionsMenu, "View/Edit Events ...").connect(SEL_COMMAND, method(:viewEventData))
     
      FXMenuTitle.new(menuBar, "&Actions", nil, actionsMenu)
      
      #Reports
      reportsMenu = FXMenuPane.new(self)
      FXMenuCommand.new(reportsMenu, "Daily Summary").connect(
            SEL_COMMAND,
            method(:showWeeklySummary))
      FXMenuTitle.new(menuBar, "&Reports", nil, reportsMenu)

      #Help
      helpMenu = FXMenuPane.new(self)
      FXMenuCommand.new(helpMenu, "&About...").connect(
            SEL_COMMAND,
            method(:showAboutDialog))
      FXMenuTitle.new(menuBar, "&Help", nil, helpMenu)
      
   end
 
   def showAboutDialog(source, selector, event)
      AboutDialog.new(self, "About Work Time Recorder").execute
   end
      
   def formatArchiveDate(aDate)
      yearString = aDate.year.to_s
      retString = 
         sprintf(
            "%0.2d%0.2d#{yearString[2..3]}",
            aDate.month,
            aDate.day)
            
      return retString
   end
   
   def createArchiveFileName
      startDate = Time.at(@timeClock.eventList.first.eventTime)
      endDate   = Time.at(@timeClock.eventList.last.eventTime)
      
      return "Events_#{formatArchiveDate(startDate)}_#{formatArchiveDate(endDate)}.xml"
   end
   
   def doNewEventFile(source, selector, event)
      if @currentActivityIndex < 0 then
         retVal = FXMessageBox::information(
                        self,
                        MBOX_OK_CANCEL,
                        "Confirm New Event File",
                        "Do you really want to clear the\ncurrent event file?")
                        
         if retVal == MBOX_CLICKED_OK then
            # move the existing events file to a file of name
            # startdate_enddate_events.xml
            if @timeClock.eventList.size > 0 then
               
               begin
                  File.rename(
                     @timeClock.eventFileName,
                     createArchiveFileName())
               rescue
                  retVal = FXMessageBox::question(
                     self,
                     MBOX_YES_NO,
                     "Unable to Archive Data",
                     "The existing event data could not be\n" +
                     "archived. Do you wish to continue (all\n" +
                     "current event data will be lost)?")
                  
                  if retVal == MBOX_CLICKED_NO then
                     return
                  end
               end
            end
                        
            begin
               evFile = File.new(@timeClock.eventFileName, "w")
               evFile.close
               
               # create a new time clock for the new event
               # file
               @timeClock = 
                  TimeClock.new(
                     @timeClock.activityFileName,
                     @timeClock.eventFileName)
                     
            rescue
               FXMessageBox::error(
                     self,
                     MBOX_OK,
                     "File Creation Error",
                     "Unable to create a new event data file")       
            end
         end
      else
         FXMessageBox::error(
               self,
               MBOX_OK,
               "Close Current Activity",
               "You must close the current activity\nbefore the event file can be reset")
      end
   end
         
   def initializeWindow
      #make sure we exit properly when the user hits the close
      #window button
      connect(SEL_CLOSE, method(:exitApplication))
      
      # handle window resizes and moves
      connect(SEL_CONFIGURE, method(:doWindowPropsChanged))
      
      workTimeRecorderProps = ApplicationProperties.instance.propertyHash["WorkTimeRecorderInfo"]
      
      self.position(
         workTimeRecorderProps["x"].to_i,
         workTimeRecorderProps["y"].to_i,
         workTimeRecorderProps["width"].to_i,
         workTimeRecorderProps["height"].to_i)
               
      windowContent = 
         FXVerticalFrame.new(
            self, 
            LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,
            0, 0, 0, 0, 0, 0, 0, 0)                  

      #initialize the current activity fields
      currentActivityGroup = 
            FXGroupBox.new(
               windowContent, 
               "Current Activity",
               GROUPBOX_TITLE_LEFT|FRAME_RIDGE|LAYOUT_FILL_X)
      currentActivityData = 
            FXMatrix.new(currentActivityGroup, 2, MATRIX_BY_COLUMNS)
            
      FXLabel.new(currentActivityData, "Name:", nil, JUSTIFY_RIGHT)
      @currentActivity = FXLabel.new(currentActivityData, "None", nil, JUSTIFY_LEFT)
      @currentActivityIndex = -1
      
      FXLabel.new(currentActivityData, "Started:", nil, JUSTIFY_RIGHT)
      @currentActivityStartTimeField = FXLabel.new(currentActivityData, "None", nil, JUSTIFY_LEFT)
      
      FXLabel.new(currentActivityData, "Hours Worked:", nil, JUSTIFY_RIGHT)
      @hoursWorkedOnCurrentActivity = FXLabel.new(currentActivityData, "None", nil, JUSTIFY_LEFT)

      stopButtonFrame = 
         FXHorizontalFrame.new(
            currentActivityGroup, 
            LAYOUT_RIGHT)

      # Stop
      FXButton.new(
            stopButtonFrame, 
            "&Stop", 
            nil, 
            nil, 
            0).connect(SEL_COMMAND, method(:stopActivity))
     
      activityListGroup = 
            FXGroupBox.new(
               windowContent, 
               "Activity List",
               GROUPBOX_TITLE_LEFT|FRAME_RIDGE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
 
      activityListGroupContent = 
         FXVerticalFrame.new(
            activityListGroup, 
            LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,
            0, 0, 0, 0, 0, 0, 0, 0)
     
      #create the activity list box
      @activityList = FXList.new(activityListGroupContent, nil, 0, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
      
      #add a right-click handler for the activity list
      @activityList.connect(SEL_RIGHTBUTTONRELEASE, method(:showListContextMenu))
      
      #add a handler that allows an activity to be started with a double click
      @activityList.connect(SEL_LEFTBUTTONRELEASE, method(:doubleClickStartActivity))
      
      #create the activity list manipulation buttons
      activityListButtons = 
            FXHorizontalFrame.new(
               activityListGroupContent, 
               LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH)
      FXButton.new(activityListButtons,"Add").connect(
            SEL_COMMAND,
            method(:addActivity))
      FXButton.new(activityListButtons,"Edit").connect(
            SEL_COMMAND,
            method(:editActivity))
      FXButton.new(activityListButtons,"Delete").connect(
            SEL_COMMAND,
            method(:deleteActivity))
 
      bottomButtonFrame = 
         FXHorizontalFrame.new(
            windowContent, 
            LAYOUT_RIGHT)
            
      FXButton.new(bottomButtonFrame, "Exit").connect(
         SEL_COMMAND,
         method(:exitApplication))

   end

   def populateActivityList
      @timeClock.activityList.each {
         |item|
         @activityList.appendItem(FXListItem.new(item))
      }
   end

   def setDefaultActivity(source, selector, event)
      if @activityList.currentItem >= 0 then
         if @timeClock.defaultActivity != nil then
            oldDefaultIndex = @activityList.findItem(@timeClock.defaultActivity)
            if oldDefaultIndex >= 0 and 
               (oldDefaultIndex != @currentActivityIndex) then
               @activityList.getItem(oldDefaultIndex).setIcon(nil)
            end
         end
         
         @timeClock.setDefaultActivity(
               @activityList.getItemText(@activityList.currentItem))
                
         if @activityList.currentItem != @currentActivityIndex then     
            @activityList.getItem(@activityList.currentItem).setIcon(
                  @defaultActivityIcon)
         end
                        
         @activityList.update
      end   
   end
   
   def showListContextMenu(source, selector, event)
      itemSelected = false
      selectedItem = nil
      
      if @activityList.currentItem >= 0 then
         itemSelected = true
         selectedItem = @activityList.getItemText(@activityList.currentItem)
      end
      
      popupMenu = FXMenuPane.new(source)
      
      if itemSelected and (@currentActivityIndex != @activityList.currentItem) then
         FXMenuCommand.new(popupMenu, "Start").connect(SEL_COMMAND,method(:startActivity))
      elsif (@currentActivityIndex == @activityList.currentItem) and
            (@currentActivityIndex != -1) then
         FXMenuCommand.new(popupMenu, "Stop").connect(SEL_COMMAND,method(:stopActivity))
      end
      
      activitySubMenu = FXMenuPane.new(source)
      
      FXMenuCommand.new(activitySubMenu, "Add").connect(SEL_COMMAND,method(:addActivity))
      
      #don't allow user to edit or delete an open activity
      if itemSelected and (@currentActivityIndex != @activityList.currentItem) then
         FXMenuCommand.new(activitySubMenu, "Edit").connect(SEL_COMMAND,method(:editActivity))
         FXMenuCommand.new(activitySubMenu, "Delete").connect(SEL_COMMAND,method(:deleteActivity))
      end

      if itemSelected and (selectedItem != @timeClock.defaultActivity) then
         FXMenuCommand.new(popupMenu, "Set As Default").connect(SEL_COMMAND,method(:setDefaultActivity))         
      end
      
      FXMenuCascade.new(popupMenu, "Activities", nil, activitySubMenu)
      
      popupMenu.create
      
      popupMenu.popup(nil, event.root_x, event.root_y)
   end
   
   def addActivity(source, selector, event)
      aDialog = ActivityDialog.new(self, "Add Activity")
      
      if aDialog.execute == 1 then
         @timeClock.addActivity(aDialog.activityName.text)
         
         @activityList.appendItem(FXListItem.new(aDialog.activityName.text))
      end     
   end

   def editActivity(source, selector, event)
      if @activityList.currentItem >= 0 then
         selectedItem = @activityList.getItemText(@activityList.currentItem)
         
         if @currentActivityIndex == @activityList.currentItem then
            FXMessageBox::error(
                  self,
                  MBOX_OK,
                  "Can't Edit Open Activity",
                  "You must close this activity before\nit can be edited")
            return
         end
         
         aDialog = ActivityDialog.new(self, "Edit Activity")
         
         aDialog.activityName.text = selectedItem
         
         if aDialog.execute == 1 then
            @timeClock.renameActivity(selectedItem, aDialog.activityName.text)

            @activityList.getItem(
                  @activityList.currentItem).text = aDialog.activityName.text                  
         end           
      end
   end

   def deleteActivity(source, selector, event)
      if @activityList.currentItem >= 0 then
         selectedItem = @activityList.getItemText(@activityList.currentItem)
         
         if @currentActivityIndex == @activityList.currentItem then
            FXMessageBox::error(
                  self,
                  MBOX_OK,
                  "Can't Delete Open Activity",
                  "You must close this activity before\nit can be deleted")
            return
         end

         @timeClock.deleteActivity(selectedItem)
         @activityList.removeItem(@activityList.currentItem)
      end   
   end
      
   def doStartActivity(selectedItem, index)
      if @activityList.currentItem >= 0 then
         updateDefaultActivityIcon
         
         @currentActivityStartTime = Time.now

         @timeClock.startActivity(selectedItem)
                  
         @currentActivity.text = selectedItem        
         
         @currentActivityIndex = index
         
         @currentActivityStartTimeField.text = formatTime(@currentActivityStartTime)
         
         @hoursWorkedOnCurrentActivity.text = formatTimeInterval(0)
         
         #Add the activity started icon to the
         #current activity
         @activityList.getItem(@currentActivityIndex).setIcon(@activeTaskIcon)
         @activityList.update
         
         updateWindowTitle
      end   
   end
   
   def updateDefaultActivityIcon
      if @currentActivityIndex >= 0 then
         if @activityList.getItemText(@currentActivityIndex) == 
            @timeClock.defaultActivity then
            @activityList.getItem(@currentActivityIndex).setIcon(@defaultActivityIcon)
         else 
            @activityList.getItem(@currentActivityIndex).setIcon(nil)
         end
      end   
      
      @activityList.update
   end
   
   def doubleClickStartActivity(source, selector, event)
      if event.click_count == 2 then
         startActivity(source, selector, event)
      end
   end
   
   def startActivity(source, selector, event)
      if @activityList.currentItem >= 0 then
         doStartActivity(
               @activityList.getItemText(@activityList.currentItem),
               @activityList.currentItem)
      end   
   end

   def stopActivity(source, selector, event)
      if @currentActivityIndex >= 0 then
         updateDefaultActivityIcon
         
         @currentActivityStartTime = nil
         @timeClock.stopActivity(@currentActivity.text)
         @currentActivity.text = "None"
         @currentActivityStartTimeField.text = "None"
         @hoursWorkedOnCurrentActivity.text = "None"
         @currentActivityIndex = -1

         self.title = @appTitle
      end   
   end
   
   def showWeeklySummary(source, selector, event)
      #get data for last week
            
      currTime = Time.now
      
      endTime = Time.local(currTime.year, currTime.mon, currTime.day).to_i
      startTime = endTime - 3600 * 24 * 7

      ReportDialog.new(self, @timeClock, startTime, endTime).execute
   end
      
   def exitApplication(source, selector, event)
      @timeClock.stopTimeClock
      
      ApplicationProperties.instance.close
      
      exit
   end   
   
   def formatTimeInterval(diffSec)      
      retString = 
         sprintf(
            "#{diffSec.to_i / 3600} hrs #{(diffSec.to_i / 60) % 60} min (%0.2f hrs)",
            diffSec / 3600.0)
            
      return retString
   end
   
   def formatTime(currTime)
      retString = 
         sprintf(
            "%0.2d/%0.2d/#{currTime.year} %0.2d:%0.2d:%0.2d",
            currTime.month,
            currTime.day,
            currTime.hour,
            currTime.min,
            currTime.sec)
            
      return retString
   end
   
   def viewEventData(source, selector, event)
      TimeEditorDialog.new(self, @timeClock.eventList).execute 
      
      @timeClock.writeEventFile
   end
   
   def workTimerCallback(source,selector,event)
      if @currentActivityStartTime != nil then      
         diffSec = Time.now - @currentActivityStartTime         
         @hoursWorkedOnCurrentActivity.text = formatTimeInterval(diffSec)

         updateWindowTitle
      end
      
      @updateTimer = self.getApp().addTimeout(@timerInterval,method(:workTimerCallback))
   end
   
   def updateWindowTitle
      diffSec = Time.now - @currentActivityStartTime         
      
      self.title = 
         sprintf(
            "#{@currentActivity.text} (%0.2f hrs)",
            diffSec / 3600.0)
   end
end

if __FILE__ == $0
  FXApp.new("Work Timer", "Mike Kelly") do |theApp|
    TimerGui.new(theApp)
    theApp.create
    theApp.run
  end
end

end